//--------------------------------------------------------------
// File     : stm32_ub_tiled_global.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_TILED_GLOBAL_H
#define __STM32F4_UB_TILED_GLOBAL_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include <stdbool.h>
#include "stm32_ub_graphic2d.h"



//--------------------------------------------------------------
// USER : ab hier keine Aenderungen notwendig
//--------------------------------------------------------------



//--------------------------------------------------------------
// Globale Struktur : Sprite
//
// speichert die x,y position von einem Sprite auf der MAP
// visible = true, falls Sprite sichtbar auf dem Screen
//--------------------------------------------------------------
typedef struct {
  uint32_t px_posx;  // position in X-Richtung (0...x)
  uint32_t px_posy;  // position in Y-Richtung (0...y)
  bool visible;      // sichtbar oder nicht (wird berechnet)
}TILED_SPRITE_t;


//--------------------------------------------------------------
// Globale Struktur : Tileset
//
// Daten von einem Tileset
// Alle Tiles muessen die gleiche groesse besitzen
// Anzahl der Tiles kann auch < sein als (anz_x * anz_y)
// width und height duerfen nicht ungerade sein
// spacing = abstand der tiles untereinander (in x und y richtung)
// transparenz farbe wird nicht auf den screen gezeichnet
//--------------------------------------------------------------
typedef struct {
  UB_Image *ptr_img;     // pointer auf das Tileset Image
  uint16_t anzx;         // anzahl tiles in x-Richtung (1...n)
  uint16_t anzy;         // anzahl tiles in y-Richtung (1...n)
  uint16_t anz;          // anzahl tiles gesammt
  uint16_t tilewidth;    // breite eines tiles in pixel (1...n)
  uint16_t tileheight;   // hoehe eines tiles in pixel (1...n)
  uint16_t spacing;      // spacing in pixel
  uint16_t transparenz;  // transparenz farbe (16bit)
}TILED_TILESET_t;


//--------------------------------------------------------------
// Globale Enumeration : Richtung
//
// dient zur richtungsangabe bei bewegungen
//--------------------------------------------------------------
typedef enum {
  DIR_NONE =0, // nicht bewegen
  DIR_UP,      // nach oben bewegen
  DIR_RIGHT,   // nach rechts bewegen
  DIR_DOWN,    // nach unten bewegen
  DIR_LEFT     // nach links bewegen
}TILED_DIR_t;


//--------------------------------------------------------------
// Globale Struktur : Tileset Animation
//
// dient fuer die Sprite Animation
// alle Richtungen mussen die gleiche Anzahl an Bildern haben
// (ausnahme = Stop)
// alle Tiles einer Richtung muessen nacheinander
// im Tileset gespeichert sein
// z.B. anzahl = 3, tile_down=6 -> dann werden die Tiles (6,7,8)
// fuer die Bewegung nach unten benutzt
//--------------------------------------------------------------
typedef struct {
  uint16_t tiles_per_animation;  // anzahl der tiles pro richtung
  uint16_t tile_stop;            // tile-nr fuer "stop"
  uint16_t tile_up;              // start tile-nr fuer "hoch"
  uint16_t tile_right;           // start tile-nr fuer "rechts"
  uint16_t tile_down;            // start tile-nr fuer "runter"
  uint16_t tile_left;            // start tile-nr fuer "links"
}TILESET_ANIMATION_t;


//--------------------------------------------------------------
// Globale Struktur : Animation
//
// speichert die aktuellen Animationsdaten von einem Sprite
//--------------------------------------------------------------
typedef struct {
  uint16_t animation_cnt;  // counter fuer die bewegung
  uint16_t tile_nr;        // aktuele tile-nr
  TILED_DIR_t akt_dir;     // aktuelle richtung
}TILED_ANIMATION_t;


//--------------------------------------------------------------
// Globale Enumeration : Fehler-Nr
//--------------------------------------------------------------
typedef enum {
  TILED_OK =0,                  // 00=kein Fehler
  WRONG_MAP_ITEM_ANZ,           // 01=Anzahl der Map-Items falsch
  TERRAIN_PARSE_ERR,            // 02=Fehler beim Parsen vom Terrain-File
  WRONG_MAP_TILE_ADR,           // 03=falsche MAP-Tile Adresse
  WRONG_MAP_TILE_NR,            // 04=falsche MAP-Tile Nr
  WRONG_TILE_ID,                // 05=falsche Tile-ID beim parsen
  WRONG_TERRAIN_TYP,            // 06=falscher Terrain-Typ beim parsen
  WRONG_TILESET_OBJ_ITEM_ANZ,   // 07=Anzahl der Tileset-Obj-Items falsch
  WRONG_ANIMATION_OBJ_ITEM_ANZ, // 08=Anzahl der Animation-Obj-Items falsch
  WRONG_SPRITE_OBJ_ITEM_ANZ,    // 09=Anzahl der Sprite-Obj-Items falsch
  WRONG_OBJ_ITEM_ANZ,           // 10=Anzahl der Obj-Items falsch
  WRONG_OBJ_TYP,                // 11=falscher Obj-Typ
  WRONG_WAYPOINT,               // 12=falscher Wegpunkt
  WRONG_PLAYER_TILE_NR,         // 13=falsche Player-Tile Nr
  WRONG_OBJ_TILE_NR,            // 14=falsche Obj-Tile Nr
  WRONG_TERRAIN_ITEM_ANZ,       // 15=Anzahl der Terrain-Items falsch
  WRONG_COLLISION_OBJ_ITEM_ANZ, // 16=Anzahl der Collisions-Typen falsch
}TILED_ERROR_t;


//--------------------------------------------------------------
// Globale Struktur : Status
//--------------------------------------------------------------
typedef struct {
  bool error;            // Error aufgetreten
  TILED_ERROR_t err_nr;  // Fehler Nr
}TILED_STATUS_t;
TILED_STATUS_t TILED_STATUS;


//--------------------------------------------------------------
// Globale Funktion
//--------------------------------------------------------------
void UB_Tiled_Global_Init(void);
void UB_Tiled_Global_Error(TILED_ERROR_t nr);


//--------------------------------------------------------------
#endif // __STM32F4_UB_TILED_GLOBAL_H
