//--------------------------------------------------------------
// File     : stm32_ub_tiled_obj.c
// Datum    : 21.09.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : TILED_MAP
// Funktion : Alle Objekte (Sprites) auf der MAP
//
// User-Einstellungen :
//  USER_O01 = Tilesets aller Objekt-Typen (.c)
//  USER_O02 = Animations-Daten aller Objekt-Typen (.c)
//  USER_O03 = Collision-Typ aller Objekt-Typen (.c)
//  USER_O04 = Liste aller Objekte (.c)
//  USER_O05 = Startpositionen aller Objekte (.c)
//  USER_O06 = Liste aller Wegpunkte (.c)
//  USER_O07 = Aufzaehlung der verschiedenen Objekt-Typen (.h)
//  USER_O08 = Images der Tileset aller Objekt-Typen (.h)
//  USER_O09 = Anzahl aller Objekte (.h)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_tiled_obj.h"




//--------------------------------------------------------------
// Tilesets aller Objekt-Typen [USER_O01]
// (abhaengig vom BMP-File des jeweilgen Tilesets)
//
// pro Objekt-Typ einen Eintrag
// Filename, anz_x, anz_y, anz, px_x, px_y, spacing, transparenz
//--------------------------------------------------------------
const TILED_TILESET_t  TILESET_OBJ[]= {
  {&Image_Knight1,2,3,8,32,32,1,0xFFFF},  // Obj = Knight1
  {&Image_Devil1,2,3,8,32,32,1,0xFFFF},   // Obj = Devil1
  {&Image_Mage1,2,3,8,32,32,1,0xFFFF},    // Obj = Mage1
  {&Image_Monster1,2,3,8,32,32,1,0xFFFF}, // Obj = Monster1
  {&Image_Items1,8,2,12,32,32,1,0xFFFF},  // Obj = Coin
  {&Image_Items1,8,2,12,32,32,1,0xF81F},  // Obj = Poison1
  {&Image_Items1,8,2,12,32,32,1,0xF81F},  // Obj = Poison2
};
const uint32_t TILESET_OBJ_ITEMS=sizeof(TILESET_OBJ)/sizeof(TILESET_OBJ[0]);


//--------------------------------------------------------------
// Animations-Daten aller Objekt-Typen [USER_O02]
// pro Objekt-Typ einen Eintrag
// Tile-Nummern vom Tileset eintragen [1...n]
//
// tiles_per_animation, stop, up, right, down, left
//--------------------------------------------------------------
const TILESET_ANIMATION_t TILESET_ANIMATION_OBJ[] = {
  {2,3,1,7,3,5},  // Obj = Knight1
  {2,3,1,7,3,5},  // Obj = Devil1
  {2,3,1,7,3,5},  // Obj = Mage1
  {2,3,1,7,3,5},  // Obj = Monster1
  {8,1,1,1,1,1},  // Obj = Coin
  {1,9,9,9,9,9},  // Obj = Poison1
  {1,10,10,10,10,10},  // Obj = Poison2
};
const uint32_t ANIMATION_OBJ_ITEMS=sizeof(TILESET_ANIMATION_OBJ)/sizeof(TILESET_ANIMATION_OBJ[0]);


//--------------------------------------------------------------
// Collision-Typ aller Objekt-Typen [USER_O03]
// OBJ_NEUTRAL     = Objekt-Typ ist neutral
// OBJ_DEADLY      = Objekt-Typ ist toedlich
// OBJ_COLLECTABLE = Objekt-Typ ist sammelbar
//--------------------------------------------------------------
const COLLISION_TYP_t COLLISION_OBJ[]= {
  OBJ_DEADLY,      // Obj = Knight1    (toedlich)
  OBJ_DEADLY,      // Obj = Devil1     (toedlich)
  OBJ_DEADLY,      // Obj = Mage1      (toedlich)
  OBJ_DEADLY,      // Obj = Monster1   (toedlich)
  OBJ_COLLECTABLE, // Obj = Coin       (sammelbar)
  OBJ_COLLECTABLE, // Obj = Poison1    (sammelbar)
  OBJ_COLLECTABLE, // Obj = Poison2    (sammelbar)
};
const uint32_t COLLISION_OBJ_ITEMS=sizeof(COLLISION_OBJ)/sizeof(COLLISION_TYP_t);


//--------------------------------------------------------------
// Liste aller Objekte und ihre Startwegpunkte [USER_O04]
// pro Objekt einen Eintrag
//
// Objekt-Typ, enable, start_wegpunkt, [distance]
//--------------------------------------------------------------
TILED_OBJ_t TILED_OBJ[] = {
  // COINS
  {TILED_OBJ_COIN,true,0,0},
  {TILED_OBJ_COIN,true,0,0},
  {TILED_OBJ_COIN,true,0,0},
  {TILED_OBJ_COIN,true,0,0},
  {TILED_OBJ_COIN,true,0,0},
  {TILED_OBJ_COIN,true,0,0},
  // POISON
  {TILED_OBJ_POISON1,true,0,0},
  {TILED_OBJ_POISON1,true,0,0},
  {TILED_OBJ_POISON2,true,0,0},
  {TILED_OBJ_POISON2,true,0,0},
  // Devil
  {TILED_OBJ_DEVIL1,true,1,0},
  {TILED_OBJ_DEVIL1,true,1,0},
  // Mage
  {TILED_OBJ_MAGE1,true,2,0},
  {TILED_OBJ_MAGE1,true,2,0},
  // Monster
  {TILED_OBJ_MONSTER1,true,3,0},
  {TILED_OBJ_MONSTER1,true,3,0},
  // Knight
  {TILED_OBJ_KNIGHT1,true,7,0},
  {TILED_OBJ_KNIGHT1,true,7,0},
};
const uint32_t TILED_OBJ_ITEMS=sizeof(TILED_OBJ)/sizeof(TILED_OBJ[0]);


//--------------------------------------------------------------
// Startpositionen aller Objekte [USER_O05]
// (nach PowerOn)
// pro Objekt einen Eintrag (Reihenfolge siehe TILED_OBJ)
//
// pos_x, pos_y, [visible]
//--------------------------------------------------------------
TILED_SPRITE_t   SPRITE_OBJ[] = {
  // COINS
  {7*TILESET_MAP_TILEWIDTH,2*TILESET_MAP_TILEHEIGHT,false},
  {37*TILESET_MAP_TILEWIDTH,14*TILESET_MAP_TILEHEIGHT,false},
  {12*TILESET_MAP_TILEWIDTH,3*TILESET_MAP_TILEHEIGHT,false},
  {38*TILESET_MAP_TILEWIDTH,9*TILESET_MAP_TILEHEIGHT,false},
  {37*TILESET_MAP_TILEWIDTH,33*TILESET_MAP_TILEHEIGHT,false},
  {19*TILESET_MAP_TILEWIDTH,13*TILESET_MAP_TILEHEIGHT,false},
  // POISON
  {6*TILESET_MAP_TILEWIDTH,7*TILESET_MAP_TILEHEIGHT,false},
  {33*TILESET_MAP_TILEWIDTH,7*TILESET_MAP_TILEHEIGHT,false},
  {6*TILESET_MAP_TILEWIDTH,18*TILESET_MAP_TILEHEIGHT,false},
  {33*TILESET_MAP_TILEWIDTH,18*TILESET_MAP_TILEHEIGHT,false},
  // Devil
  {17*TILESET_MAP_TILEWIDTH,1*TILESET_MAP_TILEHEIGHT,false},
  {23*TILESET_MAP_TILEWIDTH,22*TILESET_MAP_TILEHEIGHT,false},
  // Mage
  {21*TILESET_MAP_TILEWIDTH,7*TILESET_MAP_TILEHEIGHT,false},
  {31*TILESET_MAP_TILEWIDTH,18*TILESET_MAP_TILEHEIGHT,false},
  // Monster
  {6*TILESET_MAP_TILEWIDTH,10*TILESET_MAP_TILEHEIGHT,false},
  {27*TILESET_MAP_TILEWIDTH,7*TILESET_MAP_TILEHEIGHT,false},
  // Knight
  {6*TILESET_MAP_TILEWIDTH,16*TILESET_MAP_TILEHEIGHT,false},
  {24*TILESET_MAP_TILEWIDTH,13*TILESET_MAP_TILEHEIGHT,false},

};
const uint32_t SPRITE_OBJ_ITEMS=sizeof(SPRITE_OBJ)/sizeof(SPRITE_OBJ[0]);


//--------------------------------------------------------------
// Liste aller Wegpunkte [USER_O06]
// pro Wegpunkt einen Eintrag
// (beliebig erweiterbar)
//
// wegpunkt, richtung, distanz, schrittweite, naechster_wegpunkt
//--------------------------------------------------------------
const TILED_WAYPOINT_t TILED_WAYPOINT[] = {
  {0,DIR_NONE,0,0,0}, // Wegpunkt fuer feste Objekte (ohne Bewegung)
  //-------------------------------------------
  {1,DIR_RIGHT,8*TILESET_MAP_TILEWIDTH,4,2},
  {2,DIR_LEFT,8*TILESET_MAP_TILEWIDTH,4,1},
  //-------------------------------------------
  {3,DIR_DOWN,2*TILESET_MAP_TILEHEIGHT,4,4},
  {4,DIR_RIGHT,6*TILESET_MAP_TILEWIDTH,4,5},
  {5,DIR_UP,2*TILESET_MAP_TILEHEIGHT,4,6},
  {6,DIR_LEFT,6*TILESET_MAP_TILEWIDTH,4,3},
  //-------------------------------------------
  {7,DIR_DOWN,2*TILESET_MAP_TILEHEIGHT,4,8},
  {8,DIR_RIGHT,9*TILESET_MAP_TILEWIDTH,4,9},
  {9,DIR_UP,2*TILESET_MAP_TILEHEIGHT,4,10},
  {10,DIR_LEFT,9*TILESET_MAP_TILEWIDTH,4,7},
  //-------------------------------------------
};
const uint32_t TILED_WAYPOINT_ITEMS=sizeof(TILED_WAYPOINT)/sizeof(TILED_WAYPOINT[0]);



//--------------------------------------------------------------
// USER : ab hier keine Aenderungen notwendig
//--------------------------------------------------------------



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_Obj_DrawTile(uint16_t nr, uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
TILED_DIR_t P_Obj_Move(uint16_t nr, TILED_DIR_t direction, uint32_t distance);
TILED_DIR_t P_Obj_Move1Px(uint16_t nr, TILED_DIR_t direction);
void P_Obj_Animation(uint16_t nr, TILED_DIR_t dir);



//--------------------------------------------------------------
// init aller Objekte
//--------------------------------------------------------------
void UB_Tiled_Obj_Init(void)
{
  uint16_t nr;
  uint8_t typ;
  uint32_t waypoint;

  // check der Eintraege
  if(TILESET_OBJ_ITEMS!=TILED_OBJ_TYP_ANZ) UB_Tiled_Global_Error(WRONG_TILESET_OBJ_ITEM_ANZ);
  if(ANIMATION_OBJ_ITEMS!=TILED_OBJ_TYP_ANZ) UB_Tiled_Global_Error(WRONG_ANIMATION_OBJ_ITEM_ANZ);
  if(COLLISION_OBJ_ITEMS!=TILED_OBJ_TYP_ANZ) UB_Tiled_Global_Error(WRONG_COLLISION_OBJ_ITEM_ANZ);
  if(SPRITE_OBJ_ITEMS!=TILED_OBJ_ANZ) UB_Tiled_Global_Error(WRONG_SPRITE_OBJ_ITEM_ANZ);
  if(TILED_OBJ_ITEMS!=TILED_OBJ_ANZ) UB_Tiled_Global_Error(WRONG_OBJ_ITEM_ANZ);

  // check auf Error
  if(TILED_STATUS.error==true) return;

  //------------------------------------
  // Animation init (von allen Objekten)
  //------------------------------------
  for(nr=0;nr<TILED_OBJ_ANZ;nr++) {
    // typ bestimmen
    typ=TILED_OBJ[nr].typ;
    if(typ<TILED_OBJ_TYP_ANZ) {
      ANIMATION_OBJ[nr].animation_cnt=0;
      ANIMATION_OBJ[nr].tile_nr=TILESET_ANIMATION_OBJ[typ].tile_stop;
      ANIMATION_OBJ[nr].akt_dir=DIR_NONE;
    }
    else {
      UB_Tiled_Global_Error(WRONG_OBJ_TYP);
    }
  }


  //------------------------------------
  // Waypoint init (von allen Objekten)
  //------------------------------------
  for(nr=0;nr<TILED_OBJ_ANZ;nr++) {
    waypoint=TILED_OBJ[nr].waypoint;
    if(waypoint<TILED_WAYPOINT_ITEMS) {
      TILED_OBJ[nr].distance=TILED_WAYPOINT[waypoint].distance;
    }
    else {
      UB_Tiled_Global_Error(WRONG_WAYPOINT);
    }
  }
}


//--------------------------------------------------------------
// zeichnet alle Objekte (falls sichtbar)
// auf den Screen
//--------------------------------------------------------------
void UB_Tiled_Obj_Draw(void)
{
  uint16_t startx,endx;
  uint16_t starty,endy;
  uint8_t visible_x,visible_y;
  uint16_t nr;
  uint8_t typ;

  // check auf Error
  if(TILED_STATUS.error==true) return;

  for(nr=0;nr<TILED_OBJ_ANZ;nr++) {

    // check ob enable
    if(TILED_OBJ[nr].enable==true) {

      // typ bestimmen
      typ=TILED_OBJ[nr].typ;

      // Border (x) bestimmen
      startx=SPRITE_OBJ[nr].px_posx;
      endx=(startx+TILESET_OBJ[typ].tilewidth)-1;

      // Border (y) bestimmen
      starty=SPRITE_OBJ[nr].px_posy;
      endy=(starty+TILESET_OBJ[typ].tileheight)-1;

      // check ob Objekt sichtbar ist (x)
      visible_x=0;
      if((startx>=TILED_BORDER.px_startx) && (startx<=TILED_BORDER.px_endx)) visible_x=1;
      if((endx>=TILED_BORDER.px_startx) && (endx<=TILED_BORDER.px_endx)) visible_x=1;

      // check ob Objekt sichtbar ist (y)
      visible_y=0;
      if((starty>=TILED_BORDER.px_starty) && (starty<=TILED_BORDER.px_endy)) visible_y=1;
      if((endy>=TILED_BORDER.px_starty) && (endy<=TILED_BORDER.px_endy)) visible_y=1;

      // check ob Objekt sichtbar
      if((visible_x!=0) && (visible_y!=0)) {
        SPRITE_OBJ[nr].visible=true;
        // Objekt zeichnen
        P_Obj_DrawTile(nr, startx, starty, endx, endy);
      }
      else {
        SPRITE_OBJ[nr].visible=false;
      }
    }
  }
}


//--------------------------------------------------------------
// bewegt alle aktiven Objekte (je nach Wegpunkt)
//--------------------------------------------------------------
void UB_Tiled_Obj_MoveAll(void)
{
  uint16_t nr;
  uint32_t waypoint;

  // check auf Error
  if(TILED_STATUS.error==true) return;

  // alle Objekte durchgehen
  for(nr=0;nr<TILED_OBJ_ANZ;nr++) {
    // check ob enable
    if(TILED_OBJ[nr].enable==true) {
      // check ob naechsten Wegpunkt erreicht
      if(TILED_OBJ[nr].distance==0) {
        // aktueller Wegpunkt bestimmen
        waypoint=TILED_OBJ[nr].waypoint;
        // naechster Wegpunkt speichern
        TILED_OBJ[nr].waypoint=TILED_WAYPOINT[waypoint].next;
        // naechste Strecke speichern
        waypoint=TILED_OBJ[nr].waypoint;
        TILED_OBJ[nr].distance=TILED_WAYPOINT[waypoint].distance;
        // Objekt Animation (fuer feststehende Objekte)
        P_Obj_Animation(nr, TILED_WAYPOINT[waypoint].direction);
      }
      else {
        // weiterbewegen
        waypoint=TILED_OBJ[nr].waypoint;

        P_Obj_Move(nr, TILED_WAYPOINT[waypoint].direction, TILED_WAYPOINT[waypoint].speed);
        // distance aktuallisieren
        if(TILED_OBJ[nr].distance>=TILED_WAYPOINT[waypoint].speed) {
          TILED_OBJ[nr].distance-=TILED_WAYPOINT[waypoint].speed;
        }
        else {
          TILED_OBJ[nr].distance=0;
        }
      }
    }
  }
}


//--------------------------------------------------------------
// interne Funktion
// bewegt ein Objekt um (distance) Pixel in eine Richtung
//
// ret_wert : Richtung in der sich das Objekt bewegt hat
//--------------------------------------------------------------
TILED_DIR_t P_Obj_Move(uint16_t nr, TILED_DIR_t direction, uint32_t distance)
{
  TILED_DIR_t movement=DIR_NONE,akt_move;
  uint32_t n;

  for(n=0;n<distance;n++) {
    akt_move=P_Obj_Move1Px(nr, direction);
    if(akt_move==DIR_NONE) {
      break;
    }
    else {
      movement=akt_move;
    }
  }

  //--------------------------------------------------------------
  // Objekt Animation
  //--------------------------------------------------------------
  P_Obj_Animation(nr, movement);

  return(movement);
}


//--------------------------------------------------------------
// interne Funktion
// bewegt ein Objekt um einen Pixel in eine Richtung
// Einschrnkungen :
//   1. die Zielposition muss innerhalb der MAP liegen
//
// ret_wert : Richtung in der sich das Objekt bewegt hat
//--------------------------------------------------------------
TILED_DIR_t P_Obj_Move1Px(uint16_t nr, TILED_DIR_t direction)
{
  TILED_DIR_t movement=DIR_NONE;
  uint8_t typ;

  // typ bestimmen
  typ=TILED_OBJ[nr].typ;

  if(direction==DIR_RIGHT) {
    // check auf grenzwert
    if(SPRITE_OBJ[nr].px_posx+TILESET_OBJ[typ].tilewidth < TILED_MAP_MAXX) {
      SPRITE_OBJ[nr].px_posx++;
      movement=DIR_RIGHT;
    }
  }
  else if(direction==DIR_LEFT) {
    // check auf grenzwert
    if(SPRITE_OBJ[nr].px_posx > 0) {
      SPRITE_OBJ[nr].px_posx--;
      movement=DIR_LEFT;
    }
  }
  else if(direction==DIR_DOWN) {
    // check auf grenzwert
    if(SPRITE_OBJ[nr].px_posy+TILESET_OBJ[typ].tileheight < TILED_MAP_MAXY) {
      SPRITE_OBJ[nr].px_posy++;
      movement=DIR_DOWN;
    }
  }
  else if(direction==DIR_UP) {
    // check auf grenzwert
    if(SPRITE_OBJ[nr].px_posy > 0) {
      SPRITE_OBJ[nr].px_posy--;
      movement=DIR_UP;
    }
  }

  return(movement);
}


//--------------------------------------------------------------
// interne Funktion
// Animiert ein Objekt-Tile
//--------------------------------------------------------------
void P_Obj_Animation(uint16_t nr, TILED_DIR_t dir)
{
  uint8_t typ;

  // typ bestimmen
  typ=TILED_OBJ[nr].typ;

  if(ANIMATION_OBJ[nr].akt_dir!=dir) ANIMATION_OBJ[nr].animation_cnt=0;

  ANIMATION_OBJ[nr].animation_cnt++;
  if(ANIMATION_OBJ[nr].animation_cnt>=TILESET_ANIMATION_OBJ[typ].tiles_per_animation) ANIMATION_OBJ[nr].animation_cnt=0;

  if(dir==DIR_NONE) {
    ANIMATION_OBJ[nr].tile_nr=TILESET_ANIMATION_OBJ[typ].tile_stop+ANIMATION_OBJ[nr].animation_cnt;
  }
  else if(dir==DIR_DOWN) {
    ANIMATION_OBJ[nr].tile_nr=TILESET_ANIMATION_OBJ[typ].tile_down+ANIMATION_OBJ[nr].animation_cnt;
  }
  else if(dir==DIR_LEFT) {
    ANIMATION_OBJ[nr].tile_nr=TILESET_ANIMATION_OBJ[typ].tile_left+ANIMATION_OBJ[nr].animation_cnt;
  }
  else if(dir==DIR_UP) {
    ANIMATION_OBJ[nr].tile_nr=TILESET_ANIMATION_OBJ[typ].tile_up+ANIMATION_OBJ[nr].animation_cnt;
  }
  else if(dir==DIR_RIGHT) {
    ANIMATION_OBJ[nr].tile_nr=TILESET_ANIMATION_OBJ[typ].tile_right+ANIMATION_OBJ[nr].animation_cnt;
  }

  ANIMATION_OBJ[nr].akt_dir=dir;
}


//--------------------------------------------------------------
// interne Funktion
// zeichnet ein Objekt-Tile auf den Screen
//--------------------------------------------------------------
void P_Obj_DrawTile(uint16_t nr, uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
  uint8_t tile_x,tile_y;
  uint8_t tile_typ;
  uint16_t screen_xp,screen_yp;
  uint16_t tileset_xp,tileset_yp;
  uint16_t width,height;
  uint16_t delta;
  DMA2D_Koord koord;
  uint8_t typ;

  // typ bestimmen
  typ=TILED_OBJ[nr].typ;

  //-----------------------------------------------
  // Tiletyp bestimmen
  //-----------------------------------------------

  // tiletyp
  tile_typ=ANIMATION_OBJ[nr].tile_nr;
  // check auf Grenzwerte
  if(tile_typ<1) {
    tile_typ=1;
    UB_Tiled_Global_Error(WRONG_OBJ_TILE_NR);
  }
  if(tile_typ>TILESET_OBJ[typ].anz) {
    tile_typ=TILESET_OBJ[typ].anz;
    UB_Tiled_Global_Error(WRONG_OBJ_TILE_NR);
  }


  //-----------------------------------------------
  // Quell position bestimmen (x,y,w,h)
  //-----------------------------------------------

  // umrechnen vom tile_typ in x,y koordinate vom tileset
  tile_y=((tile_typ-1)/TILESET_OBJ[typ].anzx);
  tile_x=tile_typ-(tile_y*TILESET_OBJ[typ].anzx)-1;

  // quellposition (x) im Tileset bestimmen
  tileset_xp=(tile_x*(TILESET_OBJ[typ].tilewidth+TILESET_OBJ[typ].spacing))+TILESET_OBJ[typ].spacing;

  // quellposition (y) im Tileset bestimmen
  tileset_yp=(tile_y*(TILESET_OBJ[typ].tileheight+TILESET_OBJ[typ].spacing))+TILESET_OBJ[typ].spacing;

  // breite (w) bestimmen
  width=TILESET_OBJ[typ].tilewidth;

  // hoehe (h) bestimmen
  height=TILESET_OBJ[typ].tileheight;


  //-----------------------------------------------
  // Ziel position bestimmen (x,y)
  //-----------------------------------------------

  // zielposition (x) auf display bestimmen
  screen_xp=TILED_SCREEN_XPOS;

  // zielposition (y) auf display bestimmen
  screen_yp=TILED_SCREEN_YPOS;


  //-----------------------------------------------
  // cropping (x)
  //-----------------------------------------------
  if(TILED_BORDER.px_startx>x1) {
    delta=(TILED_BORDER.px_startx-x1);
    tileset_xp+=delta;
    width-=delta;
  }
  else {
    delta=(x1-TILED_BORDER.px_startx);
    screen_xp+=delta;
  }

  if(x2>TILED_BORDER.px_endx) {
    delta=(x2-TILED_BORDER.px_endx);
    width-=delta;
  }


  //-----------------------------------------------
  // cropping (y)
  //-----------------------------------------------
  if(TILED_BORDER.px_starty>y1) {
    delta=(TILED_BORDER.px_starty-y1);
    tileset_yp+=delta;
    height-=delta;
  }
  else {
    delta=(y1-TILED_BORDER.px_starty);
    screen_yp+=delta;
  }

  if(y2>TILED_BORDER.px_endy) {
    delta=(y2-TILED_BORDER.px_endy);
   height-=delta;
  }


  //-----------------------------------------------
  // Objekt-Tile auf Screen kopieren (mit Transparenz)
  //-----------------------------------------------
  koord.dest_xp=screen_xp;
  koord.dest_yp=screen_yp;
  koord.source_xp=tileset_xp;
  koord.source_yp=tileset_yp;
  koord.source_w=width;
  koord.source_h=height;

  UB_Graphic2D_CopySprite(TILESET_OBJ[typ].ptr_img, koord, TILESET_OBJ[typ].transparenz);
}
