//--------------------------------------------------------------
// File     : stm32_ub_tiled_player.c
// Datum    : 21.09.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : TILED_MAP
// Funktion : Spieler Srite
//
// User-Einstellungen :
//  USER_P01 = Tileset vom Player (.c)
//  USER_P02 = Animations-Daten vom Player (.c)
//  USER_P03 = Image vom Tileset vom Player (.h)
//  USER_P04 = Startposition vom Player (.h)
//  USER_P05 = Zielposition vom Player (.h)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_tiled_player.h"




//--------------------------------------------------------------
// Tileset vom Player [USER_P01]
// (abhngig vom BMP-File des Player-Tilesets)
//
// Filename, anz_x, anz_y, anz, px_x, px_y, spacing, transparenz
//--------------------------------------------------------------
const TILED_TILESET_t  TILESET_PLAYER= {
  &Image_Player,5,4,20,32,32,1,0xFFFF
};


//--------------------------------------------------------------
// Animations-Daten vom Player [USER_P02]
// Tile-Nummern vom Tileset eintragen  [1...n]
//
// tiles_per_animation, stop, up, right, down, left
//--------------------------------------------------------------
const TILESET_ANIMATION_t TILESET_ANIMATION_PLAYER = {
  5,1,11,16,1,6
};



//--------------------------------------------------------------
// USER : ab hier keine Aenderungen notwendig
//--------------------------------------------------------------



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
TILED_DIR_t P_Player_Move1Px(TILED_DIR_t direction);
void P_Player_Animation(TILED_DIR_t dir);
void P_Player_AutoMapMovement(void);
void P_Player_DrawTile(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);


//--------------------------------------------------------------
// init vom Player
//--------------------------------------------------------------
void UB_Tiled_Player_Init(void)
{
  //------------------------------------
  // Sprite init (Player)
  //------------------------------------
  SPRITE_PLAYER.px_posx=(TILED_PLAYER_STARTX*TILESET_PLAYER.tilewidth);
  SPRITE_PLAYER.px_posy=(TILED_PLAYER_STARTY*TILESET_PLAYER.tileheight);
  SPRITE_PLAYER.visible=false;

  //------------------------------------
  // Animation init (Player)
  //------------------------------------
  ANIMATION_PLAYER.animation_cnt=0;
  ANIMATION_PLAYER.tile_nr=TILESET_ANIMATION_PLAYER.tile_stop;
  ANIMATION_PLAYER.akt_dir=DIR_NONE;
}


//--------------------------------------------------------------
// zeichnet den Player (falls sichtbar)
// auf den Screen
//--------------------------------------------------------------
void UB_Tiled_Player_Draw(void)
{
  uint16_t startx,endx;
  uint16_t starty,endy;
  uint8_t visible_x,visible_y;

  // check auf Error
  if(TILED_STATUS.error==true) return;

  // Border (x) bestimmen
  startx=SPRITE_PLAYER.px_posx;
  endx=(startx+TILESET_PLAYER.tilewidth)-1;

  // Border (y) bestimmen
  starty=SPRITE_PLAYER.px_posy;
  endy=(starty+TILESET_PLAYER.tileheight)-1;

  // check ob Player sichtbar ist (x)
  visible_x=0;
  if((startx>=TILED_BORDER.px_startx) && (startx<=TILED_BORDER.px_endx)) visible_x=1;
  if((endx>=TILED_BORDER.px_startx) && (endx<=TILED_BORDER.px_endx)) visible_x=1;

  // check ob Player sichtbar ist (y)
  visible_y=0;
  if((starty>=TILED_BORDER.px_starty) && (starty<=TILED_BORDER.px_endy)) visible_y=1;
  if((endy>=TILED_BORDER.px_starty) && (endy<=TILED_BORDER.px_endy)) visible_y=1;

  // check ob Player sichtbar
  if((visible_x!=0) && (visible_y!=0)) {
    SPRITE_PLAYER.visible=true;
    // Player zeichnen
    P_Player_DrawTile(startx, starty, endx, endy);
  }
  else {
    SPRITE_PLAYER.visible=false;
  }
}


//--------------------------------------------------------------
// bewegt den Player um (distance) Pixel in eine Richtung
// (bewegt auch die MAP falls notwendig)
//
// ret_wert : Richtung in der sich der Player bewegt hat
//--------------------------------------------------------------
TILED_DIR_t UB_Tiled_Player_Move(TILED_DIR_t direction, uint32_t distance)
{
  TILED_DIR_t movement=DIR_NONE,akt_move;
  uint32_t n;

  // check auf Error
  if(TILED_STATUS.error==true) return(DIR_NONE);

  for(n=0;n<distance;n++) {
    akt_move=P_Player_Move1Px(direction);
    if(akt_move==DIR_NONE) {
      break;
    }
    else {
      movement=akt_move;
    }
  }

  //--------------------------------------------------------------
  // Player Animation
  //--------------------------------------------------------------
  P_Player_Animation(movement);

  //--------------------------------------------------------------
  // Map Bewegung
  //--------------------------------------------------------------
  if(movement!=DIR_NONE) {
    P_Player_AutoMapMovement();
  }

  return(movement);
}


//--------------------------------------------------------------
// interne Funktion
// bewegt den Player um einen Pixel in eine Richtung
// Einschraenkungen :
//   1. die Zielposition muss innerhalb der MAP liegen
//   2. das Terrain der Zielposition muss begehbar sein
//
// ret_wert : Richtung in der sich der Player bewegt hat
//--------------------------------------------------------------
TILED_DIR_t P_Player_Move1Px(TILED_DIR_t direction)
{
  TILED_DIR_t movement=DIR_NONE;
  uint32_t xp,yp;
  uint8_t terrain1, terrain2;

  if(direction==DIR_RIGHT) {
    // check auf grenzwert
    if(SPRITE_PLAYER.px_posx+TILESET_PLAYER.tilewidth < TILED_MAP_MAXX) {
      // rechte obere ecke auslesen
      xp=SPRITE_PLAYER.px_posx+TILESET_PLAYER.tilewidth;
      yp=SPRITE_PLAYER.px_posy;
      terrain1=UB_Tiled_Map_GetTerrain(xp,yp);
      // rechte untere ecke auslesen
      xp=SPRITE_PLAYER.px_posx+TILESET_PLAYER.tilewidth;
      yp=SPRITE_PLAYER.px_posy+TILESET_PLAYER.tileheight-1;
      terrain2=UB_Tiled_Map_GetTerrain(xp,yp);
      // check ob terrain begehbar
      if((PLAYER_TERRAIN[terrain1]!=TERRAIN_BLOCKED) && (PLAYER_TERRAIN[terrain2]!=TERRAIN_BLOCKED)) {
        // terrain ist begehbar
        SPRITE_PLAYER.px_posx++;
        movement=DIR_RIGHT;
      }
    }
  }
  else if(direction==DIR_LEFT) {
    // check auf grenzwert
    if(SPRITE_PLAYER.px_posx > 0) {
      // linke obere ecke auslesen
      xp=SPRITE_PLAYER.px_posx-1;
      yp=SPRITE_PLAYER.px_posy;
      terrain1=UB_Tiled_Map_GetTerrain(xp,yp);
      // linke untere ecke auslesen
      xp=SPRITE_PLAYER.px_posx-1;
      yp=SPRITE_PLAYER.px_posy+TILESET_PLAYER.tileheight-1;
      terrain2=UB_Tiled_Map_GetTerrain(xp,yp);
      // check ob terrain begehbar
      if((PLAYER_TERRAIN[terrain1]!=TERRAIN_BLOCKED) && (PLAYER_TERRAIN[terrain2]!=TERRAIN_BLOCKED)) {
        // terrain ist begehbar
        SPRITE_PLAYER.px_posx--;
        movement=DIR_LEFT;
      }
    }
  }
  else if(direction==DIR_DOWN) {
    // check auf grenzwert
    if(SPRITE_PLAYER.px_posy+TILESET_PLAYER.tileheight < TILED_MAP_MAXY) {
      // rechte untere ecke auslesen
      xp=SPRITE_PLAYER.px_posx+TILESET_PLAYER.tilewidth-1;
      yp=SPRITE_PLAYER.px_posy+TILESET_PLAYER.tileheight;
      terrain1=UB_Tiled_Map_GetTerrain(xp,yp);
      // linke untere ecke auslesen
      xp=SPRITE_PLAYER.px_posx;
      yp=SPRITE_PLAYER.px_posy+TILESET_PLAYER.tileheight;
      terrain2=UB_Tiled_Map_GetTerrain(xp,yp);
      // check ob terrain begehbar
      if((PLAYER_TERRAIN[terrain1]!=TERRAIN_BLOCKED) && (PLAYER_TERRAIN[terrain2]!=TERRAIN_BLOCKED)) {
        // terrain ist begehbar
        SPRITE_PLAYER.px_posy++;
        movement=DIR_DOWN;
      }
    }
  }
  else if(direction==DIR_UP) {
    // check auf grenzwert
    if(SPRITE_PLAYER.px_posy > 0) {
      // rechte obere ecke auslesen
      xp=SPRITE_PLAYER.px_posx+TILESET_PLAYER.tilewidth-1;
      yp=SPRITE_PLAYER.px_posy-1;
      terrain1=UB_Tiled_Map_GetTerrain(xp,yp);
      // linke obere ecke auslesen
      xp=SPRITE_PLAYER.px_posx;
      yp=SPRITE_PLAYER.px_posy-1;
      terrain2=UB_Tiled_Map_GetTerrain(xp,yp);
      // check ob terrain begehbar
      if((PLAYER_TERRAIN[terrain1]!=TERRAIN_BLOCKED) && (PLAYER_TERRAIN[terrain2]!=TERRAIN_BLOCKED)) {
        // terrain ist begehbar
        SPRITE_PLAYER.px_posy--;
        movement=DIR_UP;
      }
    }
  }

  return(movement);
}


//--------------------------------------------------------------
// interne Funktion
// Animiert den Player-Tile
//--------------------------------------------------------------
void P_Player_Animation(TILED_DIR_t dir)
{
  if(ANIMATION_PLAYER.akt_dir!=dir) ANIMATION_PLAYER.animation_cnt=0;

  ANIMATION_PLAYER.animation_cnt++;
  if(ANIMATION_PLAYER.animation_cnt>=TILESET_ANIMATION_PLAYER.tiles_per_animation) ANIMATION_PLAYER.animation_cnt=0;

  if(dir==DIR_NONE) {
    ANIMATION_PLAYER.tile_nr=TILESET_ANIMATION_PLAYER.tile_stop;
  }
  else if(dir==DIR_DOWN) {
    ANIMATION_PLAYER.tile_nr=TILESET_ANIMATION_PLAYER.tile_down+ANIMATION_PLAYER.animation_cnt;
  }
  else if(dir==DIR_LEFT) {
    ANIMATION_PLAYER.tile_nr=TILESET_ANIMATION_PLAYER.tile_left+ANIMATION_PLAYER.animation_cnt;
  }
  else if(dir==DIR_UP) {
    ANIMATION_PLAYER.tile_nr=TILESET_ANIMATION_PLAYER.tile_up+ANIMATION_PLAYER.animation_cnt;
  }
  else if(dir==DIR_RIGHT) {
    ANIMATION_PLAYER.tile_nr=TILESET_ANIMATION_PLAYER.tile_right+ANIMATION_PLAYER.animation_cnt;
  }

  ANIMATION_PLAYER.akt_dir=dir;
}


//--------------------------------------------------------------
// interne Funktion
// bewegt automatisch die MAP
// je nach Player Abstand zum Screen-Rand 
//--------------------------------------------------------------
void P_Player_AutoMapMovement(void)
{
  uint32_t distance,delta;
  uint32_t xp,yp;

  xp=TILED_MAP.px_offsetx;
  yp=TILED_MAP.px_offsety;

  // linke Kante
  distance=SPRITE_PLAYER.px_posx-xp;
  if(distance<TILED_SCREEN_BORDER) {
    delta=TILED_SCREEN_BORDER-distance;
    if(xp>=delta) {
      xp-=delta;
    }
    else {
      xp=0;
    }
    UB_Tiled_Map_Position(xp,yp);
  }
  // rechte Kante
  distance=(xp+TILED_SCREEN_MAXX)-(SPRITE_PLAYER.px_posx+TILESET_PLAYER.tilewidth);
  if(distance<TILED_SCREEN_BORDER) {
    delta=TILED_SCREEN_BORDER-distance;
    xp+=delta;
    UB_Tiled_Map_Position(xp,yp);
  }
  // obere Kante
  distance=SPRITE_PLAYER.px_posy-yp;
  if(distance<TILED_SCREEN_BORDER) {
    delta=TILED_SCREEN_BORDER-distance;
    if(yp>=delta) {
      yp-=delta;
    }
    else {
      yp=0;
    }
    UB_Tiled_Map_Position(xp,yp);
  }
  // untere Kante
  distance=(yp+TILED_SCREEN_MAXY)-(SPRITE_PLAYER.px_posy+TILESET_PLAYER.tileheight);
  if(distance<TILED_SCREEN_BORDER) {
    delta=TILED_SCREEN_BORDER-distance;
    yp+=delta;
    UB_Tiled_Map_Position(xp,yp);
  }
}


//--------------------------------------------------------------
// interne Funktion
// zeichnet den Player-Tile auf den Screen
//--------------------------------------------------------------
void P_Player_DrawTile(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
  uint8_t tile_x,tile_y;
  uint8_t tile_typ;
  uint16_t screen_xp,screen_yp;
  uint16_t tileset_xp,tileset_yp;
  uint16_t width,height;
  uint16_t delta;
  DMA2D_Koord koord;


  //-----------------------------------------------
  // Tiletyp bestimmen
  //-----------------------------------------------

  // tiletyp
  tile_typ=ANIMATION_PLAYER.tile_nr;
  // check auf Grenzwerte
  if(tile_typ<1) {
    tile_typ=1;
    UB_Tiled_Global_Error(WRONG_PLAYER_TILE_NR);
  }
  if(tile_typ>TILESET_PLAYER.anz) {
    tile_typ=TILESET_PLAYER.anz;
    UB_Tiled_Global_Error(WRONG_PLAYER_TILE_NR);
  }


  //-----------------------------------------------
  // Quell position bestimmen (x,y,w,h)
  //-----------------------------------------------

  // umrechnen vom tile_typ in x,y koordinate vom tileset
  tile_y=((tile_typ-1)/TILESET_PLAYER.anzx);
  tile_x=tile_typ-(tile_y*TILESET_PLAYER.anzx)-1;

  // quellposition (x) im Tileset bestimmen
  tileset_xp=(tile_x*(TILESET_PLAYER.tilewidth+TILESET_PLAYER.spacing))+TILESET_PLAYER.spacing;

  // quellposition (y) im Tileset bestimmen
  tileset_yp=(tile_y*(TILESET_PLAYER.tileheight+TILESET_PLAYER.spacing))+TILESET_PLAYER.spacing;

  // breite (w) bestimmen
  width=TILESET_PLAYER.tilewidth;

  // hoehe (h) bestimmen
  height=TILESET_PLAYER.tileheight;


  //-----------------------------------------------
  // Ziel position bestimmen (x,y)
  //-----------------------------------------------

  // zielposition (x) auf display bestimmen
  screen_xp=TILED_SCREEN_XPOS;

  // zielposition (y) auf display bestimmen
  screen_yp=TILED_SCREEN_YPOS;


  //-----------------------------------------------
  // cropping (x)
  //-----------------------------------------------
  if(TILED_BORDER.px_startx>x1) {
    delta=(TILED_BORDER.px_startx-x1);
    tileset_xp+=delta;
    width-=delta;
  }
  else {
    delta=(x1-TILED_BORDER.px_startx);
    screen_xp+=delta;
  }

  if(x2>TILED_BORDER.px_endx) {
    delta=(x2-TILED_BORDER.px_endx);
    width-=delta;
  }


  //-----------------------------------------------
  // cropping (y)
  //-----------------------------------------------
  if(TILED_BORDER.px_starty>y1) {
    delta=(TILED_BORDER.px_starty-y1);
    tileset_yp+=delta;
    height-=delta;
  }
  else {
    delta=(y1-TILED_BORDER.px_starty);
    screen_yp+=delta;
  }

  if(y2>TILED_BORDER.px_endy) {
    delta=(y2-TILED_BORDER.px_endy);
    height-=delta;
  }


  //-----------------------------------------------
  // Player-Tile auf Screen kopieren (mit Transparenz)
  //-----------------------------------------------
  koord.dest_xp=screen_xp;
  koord.dest_yp=screen_yp;
  koord.source_xp=tileset_xp;
  koord.source_yp=tileset_yp;
  koord.source_w=width;
  koord.source_h=height;

  UB_Graphic2D_CopySprite(TILESET_PLAYER.ptr_img, koord, TILESET_PLAYER.transparenz);
}
