//--------------------------------------------------------------
// File     : stm32_ub_tiled_terrain.c
// Datum    : 21.09.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : TILED_MAP
// Funktion : Parser fuer Terrain-File aus dem Flash
//
// User-Einstellungen :
//  USER_T01 = Terrain Daten fuer Player (.c)
//  USER_T02 = Terrain-File der MAP (.h)
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_tiled_terrain.h"
#include "stm32_ub_tiled_map.h"



//--------------------------------------------------------------
// Terrain Daten fuer Player [USER_T01]
// TERRAIN_BLOCKED = Terrain ist fuer Player gesperrt
// TERRAIN_FREE    = Terrain ist fuer Player begehbar
// TERRAIN_DEADLY  = Terrain ist fuer Player toedlich
//
// Reihenfolge der Typen siehe "MAP.tsx" -> "terraintypes"
//--------------------------------------------------------------
const TERRAIN_TYP_t PLAYER_TERRAIN[]= {
  TERRAIN_FREE,    // typ-0 = dessert     (begehbar)
  TERRAIN_BLOCKED, // typ-1 = brick       (gesperrt)
  TERRAIN_BLOCKED, // typ-2 = cobblestone (gesperrt)
  TERRAIN_FREE     // typ-3 = dirt        (begehbar)
};
const uint32_t PLAYER_TERRAIN_ITEMS=sizeof(PLAYER_TERRAIN)/sizeof(TERRAIN_TYP_t);



//--------------------------------------------------------------
// USER : ab hier keine Aenderungen notwendig
//--------------------------------------------------------------



//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
static uint32_t akt_adr; // aktuelle Adresse im Flash
static uint32_t akt_pos; // aktuelle Position 0...max_pos
static uint32_t max_pos; // maximum Position


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
uint8_t P_Terrain_Search_Text(char *text);
uint16_t P_Terrain_Get_4ziff_dez(uint8_t start_char, uint8_t end_char);



//--------------------------------------------------------------
// parst das TSX-File
// fuellt die Terrain-Werte auf
//
// Beispiel einer Zeile :
//  <tile id="15" terrain="3,0,3,0"/>
//
// ret_wert : 0 = kein Fehler, alle Terrain Werte gefunden
//            1 = nicht alle Terrain Werte gefunden
//--------------------------------------------------------------
uint8_t UB_Tiled_Terrain_Parse_TSX(UB_File *file)
{
  uint8_t ret_wert=0;
  uint8_t check;
  uint16_t tile_id,tile_cnt;
  uint16_t a,b,c,d;

  // check der TERRAIN-Eintraege
  if(PLAYER_TERRAIN_ITEMS!=TERRAIN_MAP_TYPES) UB_Tiled_Global_Error(WRONG_TERRAIN_ITEM_ANZ);

  // Startadresse setzen
  max_pos=file->size;
  akt_adr=(uint32_t)&file->table[0];

  akt_pos=0;
  tile_cnt=0;
  while(akt_pos<max_pos) {
    check=P_Terrain_Search_Text("<tile id=");  // suche nach Tile-ID
    if(check>0) {
      // Terrain Werte auslesen
      tile_id=P_Terrain_Get_4ziff_dez('"','"');  // lese Tile-ID
      a=P_Terrain_Get_4ziff_dez('"',',');        // lese Terrain upper_left
      b=P_Terrain_Get_4ziff_dez(0,',');          // lese Terrain upper_right
      c=P_Terrain_Get_4ziff_dez(0,',');          // lese Terrain lower_left
      d=P_Terrain_Get_4ziff_dez(0,'"');          // lese Terrain lower_right

      // speichert Terrain-Werte
      if(tile_id<TILESET_MAP_ANZ) {
        if((a<TERRAIN_MAP_TYPES) && (b<TERRAIN_MAP_TYPES) && (c<TERRAIN_MAP_TYPES) && (d<TERRAIN_MAP_TYPES)) {
          tile_cnt++;
          TILED_TERRAIN[tile_id].upper_left=a;
          TILED_TERRAIN[tile_id].upper_right=b;
          TILED_TERRAIN[tile_id].lower_left=c;
          TILED_TERRAIN[tile_id].lower_right=d;
        }
        else {
          UB_Tiled_Global_Error(WRONG_TERRAIN_TYP);
        }
      }
      else {
        UB_Tiled_Global_Error(WRONG_TILE_ID);
      }
    }
  }

  if(tile_cnt!=TILESET_MAP_ANZ) ret_wert=1;

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// sucht eine Zeichenfolge im File
//
// ret_wert : 0 = text nicht gefunden
//            1 = text gefunden
//--------------------------------------------------------------
uint8_t P_Terrain_Search_Text(char *text)
{
  uint8_t wert,laenge,pos;

  laenge=0;
  do {
    wert=text[laenge];
    laenge++;
  }while(wert!=0);
  laenge--;

  pos=0;
  while(akt_pos<max_pos) {
    // zeichen an adresse auslesen
    wert=*(volatile uint8_t*)(akt_adr);
    // adresse weiterschalten
    akt_adr++;
    akt_pos++;

    if(wert==text[pos]) {
      // zeichen stimmt
      pos++;
      if(pos==laenge) {
        // text gefunden
        return(1);
      }
    }
    else {
      pos=0;
    }
  }

  // text nicht gefunden
  return(0);
}


//--------------------------------------------------------------
// interne Funktion
// wandelt einen string in eine Zahl um
// String kann "0" bis "9999" sein
//
// start_char : trennungszeichen (0=zahl beginnt sofort)
// end_char   : trennungszeichen
// ret_wert : 0 bis 9999
//--------------------------------------------------------------
uint16_t P_Terrain_Get_4ziff_dez(uint8_t start_char, uint8_t end_char)
{
  uint16_t ret_wert=0;
  uint8_t wert,mode;
  uint8_t ziffer,z[4];

  mode=0;
  if(start_char==0) mode=1; // zahl beginnt sofort
  ziffer=0;
  while((akt_pos<max_pos) && (mode!=2)) {
    // zeichen an adresse auslesen
    wert=*(volatile uint8_t*)(akt_adr);
    // adresse weiterschalten
    akt_adr++;
    akt_pos++;

    if(mode==0) {
      if(wert==start_char) mode=1; // start_zeichen gefunden
    }
    else {
      if(wert==end_char) {   // ende_zeichen gefunden
        mode=2;
      }
      else if((wert>='0') && (wert<='9')) {
        z[ziffer]=wert-'0';  // ziffer speichern
        ziffer++;
      }
    }
  }

  // umwandeln der Ziffern in Zahl
  if(ziffer==1) {
    ret_wert=z[0];
  }else if(ziffer==2) {
    ret_wert=z[0]*10;
    ret_wert+=z[1];
  }else if(ziffer==3) {
    ret_wert=z[0]*100;
    ret_wert+=z[1]*10;
    ret_wert+=z[2];
  }else if(ziffer==4) {
    ret_wert=z[0]*1000;
    ret_wert+=z[1]*100;
    ret_wert+=z[2]*10;
    ret_wert+=z[3];
  }

  return(ret_wert);
}




